<!DOCTYPE html>
<html>
<head>
<title>Welcome to {{NAME}}!</title>
<style>
    body {
        width: 35em;
        margin: 0 auto;
        font-family: Tahoma, Verdana, Arial, sans-serif;
    }
</style>
</head>
<body>
<h1>Welcome to {{NAME}}!</h1>
<p>The {{NAME}} version {{VERSION}}server was configured on this host at {{DATETIME}}.</p>

<p>For online documentation and support please refer to the
<a href="{{PROJECT_HOME}}">{{NAME}} site</a>.<br/>

Here are some public targets supplied by lorax:
<a href="status">{{NAME}} status</a>.<br/>
<a href="healthcheck">{{NAME}} healthcheck</a>.<br/>
<a href="healthcheck">{{NAME}} healthcheck</a>.<br/>

<p><em>Thank you for using {{NAME}}.</em></p>
</body>
</html>
